#! /usr/bin/python


##   VGB-GUI
##   Copyright (C) 2000 Maxximum Software
   
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
  
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
  
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



from gtk import *
from gnome import config
from gnome.ui import *
from GDK import *
from libglade import *

import os
import os.path
import shutil
import thread
import glob
import string
import tempfile
import gzip
import sys

# Global defs
progname = 'VGB-GUI'
version  = '0.3.1'

TRUE  = 1
FALSE = 0

DEBUG = FALSE

glade_locs = ['./', 'to be set in main', '/usr/share/vgb-gui' + '-' + version + '/']

config_opts = {'rom_path':[], 'gb_emu':'gnuboy', 'savegame_location':''}

vgb_config_opts = {'vgb_opt_uperiod':'1', 'vgb_opt_gb_type':'-cgb', 'vgb_opt_sound':'44000',
                   'vgb_opt_sync':'60', 'vgb_opt_scale':'2', 'vgb_location':'/usr/local/bin/vgb',
                   'vgb_opt_autofire_A':'-noautoa', 'vgb_opt_autofire_B':'-noautob',
                   'vgb_opt_pocket_printer':'-noprinter', 'vgb_opt_washout':'-nowashout',
                   'vgb_opt_shm':'-shm', 'vgb_opt_save_cpu':'-saver'}

gnuboy_config_opts = {'gnuboy_location':'/usr/local/bin/xgnuboy', 'gnuboy_auto_gunzip':TRUE,
                      'gnuboy_scale':'1', 'gnuboy_density':'1'}

def debug_print(printme):
    if (DEBUG==TRUE):
        print '***' + printme

def file_is_gzip (fname):
    romfile_ext = os.path.splitext (fname) [1]
    return ((romfile_ext == '.gz') \
            or (romfile_ext == '.GZ') \
            or (romfile_ext == '.gZ') \
            or (romfile_ext == '.Gz'))

def file_exists (fname):
    try:
        f = open (fname) 
        returnme = TRUE
        f.close ()
    except (IOError):
        return FALSE
    
    return returnme

def get_rom_name (fname):
    if (file_is_gzip (fname)):
        f = gzip.open (fname)
    else:
        f = open (fname)

    s = f.read (16384)

    romname = s[0x0134:0x0134+16]
    if (romname[14] and 0x80):
        romname = romname[0:15]
    elif (romname[15] and 0x80):
        romname = romname[0:16]

    returnme = romname

    f.close ()

    return returnme

def runGB(args, romfilename):
    gnuboy_prefix_opts = ''
    tmpfile = ''

    if (config_opts['gb_emu'] == 'gnuboy'):
        # If the romfile is gzipped then gunzip and pipe into gnuboy
        if  file_is_gzip (romfilename):
            if (gnuboy_config_opts['gnuboy_auto_gunzip']):
                # gunzip < romname.gb.gz | gnuboy --savename=romname /dev/stdin
                gnuboy_prefix_opts = 'gunzip < ' + '"' + romfilename + '"' + ' | '

                # Get rid of the .gz extension, but keep the .gb/.gbc extension for compatibility naming with VGB
                tmp = os.path.basename (os.path.splitext (romfilename) [0])

                args = args + ' --savename=' + '"' + os.path.basename (tmp) + '"'
                romfilename = '/dev/stdin'

        if (romfilename!=''):
            args = args + ' ' + '"' + romfilename + '"'

        debug_print (gnuboy_prefix_opts + gnuboy_config_opts['gnuboy_location'] + ' ' + args)

        os.system(gnuboy_prefix_opts + gnuboy_config_opts['gnuboy_location'] + ' ' + args)
                    
    elif (config_opts['gb_emu'] == 'VGB'):
        if (romfilename!=''):
            args = args + ' ' + '"' + romfilename + '"'
        os.system(vgb_config_opts['vgb_location'] + ' ' + args)

    thread.exit()
    
def buildArgs():
    args = ''

    if (config_opts['gb_emu'] == 'gnuboy'):
        if (config_opts['savegame_location']  != ''):
            args = args + ' --savedir=' + '"' + config_opts['savegame_location'] + '"'
        if (gnuboy_config_opts['gnuboy_scale']  != ''):
            args = args + ' --scale=' + gnuboy_config_opts['gnuboy_scale']
        if (gnuboy_config_opts['gnuboy_density']  != ''):
            args = args + ' --density=' + gnuboy_config_opts['gnuboy_density']
    elif (config_opts['gb_emu'] == 'VGB'):
        if (vgb_config_opts['vgb_opt_uperiod']!=''):
            args = args + ' -uperiod ' + vgb_config_opts['vgb_opt_uperiod']
        if (vgb_config_opts['vgb_opt_gb_type']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_gb_type']
        if (vgb_config_opts['vgb_opt_autofire_A']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_autofire_A']
        if (vgb_config_opts['vgb_opt_autofire_B']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_autofire_B']
        if (vgb_config_opts['vgb_opt_pocket_printer']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_pocket_printer']
        if (vgb_config_opts['vgb_opt_washout']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_washout']
        if (vgb_config_opts['vgb_opt_shm']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_shm']
        if (vgb_config_opts['vgb_opt_save_cpu']!=''):
            args = args + ' ' + vgb_config_opts['vgb_opt_save_cpu']
        if (vgb_config_opts['vgb_opt_sound']!=''):
            args = args + ' -sound ' + vgb_config_opts['vgb_opt_sound']
        if (vgb_config_opts['vgb_opt_sync']!=''):
            args = args + ' -sync ' + vgb_config_opts['vgb_opt_sync']
        if (vgb_config_opts['vgb_opt_scale']!=''):
            args = args + ' -scale ' + vgb_config_opts['vgb_opt_scale']

    debug_print(args)
    return args

def initApp():
    global wTree

    initialized = FALSE
    for e in glade_locs:
        debug_print ('trying: ' + e + 'vgb-gui.glade')
        if (file_exists (e + 'vgb-gui.glade') and not initialized):
            wTree = GladeXML (e + 'vgb-gui.glade', 'main_window')
            glade_loc = e
            initialized = TRUE
            debug_print ('using: '+ e + 'vgb-gui.glade')

    if not initialized:
        print "Unable to locate vgb-gui.glade"
        sys.exit ()

    dic = {'on_main_window_destroy': mainquit,
           'on_exit1_activate': mainquit,
           'on_about1_activate': on_about1_activate,
           'on_preferences1_activate': on_preferences1_activate,
           'on_refresh_activate': on_refresh_activate,
           'on_games_clist_select_row': on_games_clist_select_row}
    wTree.signal_autoconnect(dic)

def on_about1_activate(obj):
    # FIXME - I am now making the about dialog myself, since I can't figure out how to pass vesion information to GladeXML
    #about = GladeXML(glade_loc + 'vgb-gui.glade', 'about1')

    about = GnomeAbout (progname, version, '(C) 2000-2001 Maxximum Software', ['Mads Villadsen'], 'A program for launching and configuring gnuboy and VGB')
    about.show()

def path_fileselection_on_ok_button_clicked(obj):
    new_rom_path = os.path.dirname(path_fileselection.get_widget('path_fileselection').get_filename())
    debug_print(new_rom_path)
    print config_opts['rom_path']
    config_opts['rom_path'].append(new_rom_path)
    
    path_fileselection.get_widget('path_fileselection').destroy()

    row = [None] * 1
    tmp = prefs.get_widget('rom_path_clist')
    tmp.clear()
    for path in config_opts['rom_path']:
        debug_print(path)
        row[0] = path
        tmp.append(row)

def path_fileselection_on_cancel_button_clicked(obj):
    path_fileselection.get_widget('path_fileselection').destroy()

def on_add_path_button_clicked(obj):
    global path_fileselection
    path_fileselection = GladeXML(glade_loc + 'vgb-gui.glade', 'path_fileselection')
    dic = {'path_fileselection_on_ok_button_clicked'     : path_fileselection_on_ok_button_clicked,
           'path_fileselection_on_cancel_button_clicked' : path_fileselection_on_cancel_button_clicked}
    path_fileselection.signal_autoconnect(dic)
    
def on_remove_path_button_clicked(obj):
    tmp = prefs.get_widget('rom_path_clist')
    selecteditem = tmp.selection

    del config_opts['rom_path'][selecteditem[0]]

    row = [None] * 1
    tmp = prefs.get_widget('rom_path_clist')
    tmp.clear()
    for path in config_opts['rom_path']:
        debug_print(path)
        row[0] = path
        tmp.append(row)
    
def on_preferences1_activate(obj):
    global prefs
    prefs = GladeXML(glade_loc + 'vgb-gui.glade', 'prefs_dialog')
    dic = {'on_prefs_ok_button_clicked'     : on_prefs_ok_button_clicked,
           'on_prefs_cancel_button_clicked' : on_prefs_cancel_button_clicked,
           'on_add_path_button_clicked'     : on_add_path_button_clicked,
           'on_remove_path_button_clicked'  : on_remove_path_button_clicked}
    prefs.signal_autoconnect(dic)

    tmp = prefs.get_widget('gb_type_optionmenu')
    if (vgb_config_opts['vgb_opt_gb_type']=='-gb'):
        tmp.set_history(0)
    elif (vgb_config_opts['vgb_opt_gb_type']=='-sgb'):
        tmp.set_history(1)
    elif (vgb_config_opts['vgb_opt_gb_type']=='-cgb'):
        tmp.set_history(2)
    elif (vgb_config_opts['vgb_opt_gb_type']=='-mgb'):
        tmp.set_history(3)

    tmp = prefs.get_widget('uperiod_spinbutton')
    tmp.set_value(int(vgb_config_opts['vgb_opt_uperiod']))

    tmp = prefs.get_widget('sync_spinbutton')
    tmp.set_value(int(vgb_config_opts['vgb_opt_sync']))

    tmp = prefs.get_widget('sound_spinbutton')
    tmp.set_value(int(vgb_config_opts['vgb_opt_sound']))
    
    tmp = prefs.get_widget('scale_spinbutton')
    tmp.set_value(int(vgb_config_opts['vgb_opt_scale']))

    tmp = prefs.get_widget('savegame_location_fileentry')
    tmp2 = tmp.gtk_entry()
    tmp2.set_text(config_opts['savegame_location'])

    tmp = prefs.get_widget('vgb_location_fileentry')
    tmp2 = tmp.gtk_entry()
    tmp2.set_text(vgb_config_opts['vgb_location'])

    tmp = prefs.get_widget('autofire_A_checkbutton')
    if (vgb_config_opts['vgb_opt_autofire_A']=='-noautoa'):
        tmp.set_active(0)
    elif (vgb_config_opts['vgb_opt_autofire_A']=='-autoa'):
        tmp.set_active(1)

    tmp = prefs.get_widget('autofire_B_checkbutton')
    if (vgb_config_opts['vgb_opt_autofire_B']=='-noautob'):
        tmp.set_active(0)
    elif (vgb_config_opts['vgb_opt_autofire_B']=='-autob'):
        tmp.set_active(1)

    tmp = prefs.get_widget('pocket_printer_checkbutton')
    if (vgb_config_opts['vgb_opt_pocket_printer']=='-noprinter'):
        tmp.set_active(0)
    elif (vgb_config_opts['vgb_opt_pocket_printer']=='-printer'):
        tmp.set_active(1)
    tmp = prefs.get_widget('washout_checkbutton')
    if (vgb_config_opts['vgb_opt_washout']=='-nowashout'):
        tmp.set_active(0)
    elif (vgb_config_opts['vgb_opt_washout']=='-washout'):
        tmp.set_active(1)
    tmp = prefs.get_widget('shm_checkbutton')
    if (vgb_config_opts['vgb_opt_shm']=='-noshm'):
        tmp.set_active(0)
    elif (vgb_config_opts['vgb_opt_shm']=='-shm'):
        tmp.set_active(1)
    tmp = prefs.get_widget('save_cpu_checkbutton')
    if (vgb_config_opts['vgb_opt_save_cpu']=='-nosaver'):
        tmp.set_active(0)
    elif (vgb_config_opts['vgb_opt_save_cpu']=='-saver'):
        tmp.set_active(1)



    tmp = prefs.get_widget('gnuboy_location_fileentry')
    tmp2 = tmp.gtk_entry()
    tmp2.set_text(gnuboy_config_opts['gnuboy_location'])

    tmp = prefs.get_widget('gnuboy_gunzip_checkbutton')
    if (gnuboy_config_opts['gnuboy_auto_gunzip']):
        tmp.set_active(1)
    else:
        tmp.set_active(0)

    tmp = prefs.get_widget('gnuboy_scale_optionmenu')
    tmp.set_history(int(gnuboy_config_opts['gnuboy_scale'])-1)

    tmp = prefs.get_widget('gnuboy_density_optionmenu')
    tmp.set_history(int(gnuboy_config_opts['gnuboy_density'])-1)



    row = [None] * 1
    tmp = prefs.get_widget('rom_path_clist')
    tmp.clear()
    for path in config_opts['rom_path']:
        debug_print(path)
        row[0] = path
        tmp.append(row)

    tmp = prefs.get_widget('gb_emu_optionmenu')
    if (config_opts['gb_emu']=='VGB'):
        tmp.set_history(0)
    elif (config_opts['gb_emu']=='gnuboy'):
        tmp.set_history(1)


def on_prefs_ok_button_clicked(obj):
    tmp = prefs.get_widget('gb_type_optionmenu')
    tmp_menu = tmp.get_menu()
    tmp_active = tmp_menu.get_active()
    tmp_index = tmp_menu.children().index(tmp_active)
    if (tmp_index == 0):
        vgb_config_opts['vgb_opt_gb_type'] = '-gb'
    elif (tmp_index == 1):
        vgb_config_opts['vgb_opt_gb_type'] = '-sgb'
    elif (tmp_index == 2):
        vgb_config_opts['vgb_opt_gb_type'] = '-cgb'
    elif (tmp_index == 3):
        vgb_config_opts['vgb_opt_gb_type'] = '-mgb'

    tmp = prefs.get_widget('uperiod_spinbutton')
    vgb_config_opts['vgb_opt_uperiod'] = string.split(str(tmp.get_value()), '.')[0]

    tmp = prefs.get_widget('sync_spinbutton')
    vgb_config_opts['vgb_opt_sync'] = string.split(str(tmp.get_value()), '.')[0]

    tmp = prefs.get_widget('sound_spinbutton')
    vgb_config_opts['vgb_opt_sound'] = string.split(str(tmp.get_value()), '.')[0]
    
    tmp = prefs.get_widget('scale_spinbutton')
    vgb_config_opts['vgb_opt_scale'] = string.split(str(tmp.get_value()), '.')[0]

    tmp = prefs.get_widget('savegame_location_fileentry')
    tmp2 = tmp.gtk_entry()
    config_opts['savegame_location'] = tmp2.get_text()

    tmp = prefs.get_widget('vgb_location_fileentry')
    tmp2 = tmp.gtk_entry()
    vgb_config_opts['vgb_location'] = tmp2.get_text()

    tmp = prefs.get_widget('autofire_A_checkbutton')
    if (tmp.get_active() == 0):
        vgb_config_opts['vgb_opt_autofire_A'] = '-noautoa'
    elif (tmp.get_active() == 1):
        vgb_config_opts['vgb_opt_autofire_A'] = '-autoa'

    tmp = prefs.get_widget('autofire_B_checkbutton')
    if (tmp.get_active() == 0):
        vgb_config_opts['vgb_opt_autofire_B'] = '-noautob'
    elif (tmp.get_active() == 1):
        vgb_config_opts['vgb_opt_autofire_B'] = '-autob'

    tmp = prefs.get_widget('pocket_printer_checkbutton')
    if (tmp.get_active() == 0):
        vgb_config_opts['vgb_opt_pocket_printer'] = '-noprinter'
    elif (tmp.get_active() == 1):
        vgb_config_opts['vgb_opt_pocket_printer'] = '-printer'
    tmp = prefs.get_widget('washout_checkbutton')
    if (tmp.get_active() == 0):
        vgb_config_opts['vgb_opt_washout'] = '-nowashout'
    elif (tmp.get_active() == 1):
        vgb_config_opts['vgb_opt_washout'] = '-washout'
    tmp = prefs.get_widget('shm_checkbutton')
    if (tmp.get_active() == 0):
        vgb_config_opts['vgb_opt_shm'] = '-noshm'
    elif (tmp.get_active() == 1):
        vgb_config_opts['vgb_opt_shm'] = '-shm'
    tmp = prefs.get_widget('save_cpu_checkbutton')
    if (tmp.get_active() == 0):
        vgb_config_opts['vgb_opt_save_cpu'] = '-nosaver'
    elif (tmp.get_active() == 1):
        vgb_config_opts['vgb_opt_save_cpu'] = '-saver'


    tmp = prefs.get_widget('gnuboy_location_fileentry')
    tmp2 = tmp.gtk_entry()
    gnuboy_config_opts['gnuboy_location'] = tmp2.get_text()

    tmp = prefs.get_widget('gnuboy_gunzip_checkbutton')
    gnuboy_config_opts['gnuboy_auto_gunzip'] = tmp.get_active ()

    tmp = prefs.get_widget('gnuboy_scale_optionmenu')
    tmp_menu = tmp.get_menu()
    tmp_active = tmp_menu.get_active()
    tmp_index = tmp_menu.children().index(tmp_active)
    gnuboy_config_opts['gnuboy_scale'] = str(tmp_index + 1)

    tmp = prefs.get_widget('gnuboy_density_optionmenu')
    tmp_menu = tmp.get_menu()
    tmp_active = tmp_menu.get_active()
    tmp_index = tmp_menu.children().index(tmp_active)
    gnuboy_config_opts['gnuboy_density'] = str(tmp_index + 1)

    # config_opts['rom_path'] is always up to date - so no need to read it back in here

    tmp = prefs.get_widget('gb_emu_optionmenu')
    tmp_menu = tmp.get_menu()
    tmp_active = tmp_menu.get_active()
    tmp_index = tmp_menu.children().index(tmp_active)
    if (tmp_index == 0):
        config_opts['gb_emu'] = 'VGB'
    elif (tmp_index == 1):
        config_opts['gb_emu'] = 'gnuboy'

    prefs.get_widget('prefs_dialog').destroy()
    save_prefs()

def on_prefs_cancel_button_clicked(obj):
    prefs.get_widget('prefs_dialog').destroy()

    
def on_games_clist_select_row(self, row, col, gdkevent):
    if (gdkevent.button == 1):
        if (gdkevent.type == GDK._2BUTTON_PRESS):
            # Execute gb emulator with selected rom
            romfname = self.get_text(row, col)

            if config_opts['savegame_location']!='':
                # The user wants the savegames in a special directory
                # Make a symlink for vgb, and start with that instead
                debug_print (config_opts['savegame_location'])
                symlinkname = config_opts['savegame_location']+'/'+os.path.basename(romfname)

                if (os.path.islink(symlinkname)): # Remove the symlink if it already exists
                    os.remove(symlinkname)        

                os.symlink(romfname, symlinkname)
                romfname = symlinkname

            opt_args = buildArgs()
            thread.start_new_thread(runGB, (opt_args, romfname))

def on_refresh_activate(obj):
    add_gb_roms_to_games_list()
            
def list_gb_roms(path):
    appbar = wTree.get_widget('appbar1')
    appbar.set_progress(0.0)

    gbroms    = glob.glob(path + '/*.[Gg][Bb]')
    appbar.set_progress(appbar.get_progress().get_current_percentage() + 1.0/4)
    gbgzroms  = glob.glob(path + '/*.[Gg][Bb].[gG][zZ]')
    appbar.set_progress(appbar.get_progress().get_current_percentage() + 1.0/4)
    gbcroms   = glob.glob(path + '/*.[Gg][Bb][cC]')
    appbar.set_progress(appbar.get_progress().get_current_percentage() + 1.0/4)
    gbcgzroms = glob.glob(path + '/*.[Gg][Bb][cC].[gG][zZ]')
    appbar.set_progress(appbar.get_progress().get_current_percentage() + 1.0/4)

    returnme = gbroms + gbgzroms + gbcroms + gbcgzroms
    returnme.sort()
    return returnme
    
def add_gb_roms_to_games_list():
    games_clist = wTree.get_widget('games_clist')
    appbar = wTree.get_widget('appbar1')

    gb_roms = []
    for path in config_opts['rom_path']:
        debug_print(path)
        gb_roms = gb_roms + list_gb_roms(path)
    
    l = len(gb_roms)
    appbar.set_progress(0.0)

    row = [None] * 2
    games_clist.clear()
    games_clist.freeze()
    for x in gb_roms:
        row[0] = get_rom_name (x)
        row[1] = x

        games_clist.append(row)

        # FIXME - appbar doesn't update...
        if (appbar.get_progress().get_current_percentage() + 1.0/l) > 1.0:
            appbar.set_progress(1.0)
        else:
            appbar.set_progress(appbar.get_progress().get_current_percentage() + 1.0/l)
            
    games_clist.thaw()
    appbar.set_progress(1.0)
    appbar.set_progress(0.0)
    appbar.push('Number of ROMS: ' + str(l))

def load_prefs():
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_uperiod')
    if (tmp != None):
        vgb_config_opts['vgb_opt_uperiod'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_gb_type')
    if (tmp != None):
        vgb_config_opts['vgb_opt_gb_type'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_sound')
    if (tmp != None):
        vgb_config_opts['vgb_opt_sound'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_sync')
    if (tmp != None):
        vgb_config_opts['vgb_opt_sync'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_scale')
    if (tmp != None):
        vgb_config_opts['vgb_opt_scale'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_location')
    if (tmp != None):
        vgb_config_opts['vgb_location'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_autofire_A')
    if (tmp != None):
        vgb_config_opts['vgb_opt_autofire_A'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_autofire_B')
    if (tmp != None):
        vgb_config_opts['vgb_opt_autofire_B'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_pocket_printer')
    if (tmp != None):
        vgb_config_opts['vgb_opt_pocket_printer'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_washout')
    if (tmp != None):
        vgb_config_opts['vgb_opt_washout'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_shm')
    if (tmp != None):
        vgb_config_opts['vgb_opt_shm'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/vgb_opt_save_cpu')
    if (tmp != None):
        vgb_config_opts['vgb_opt_save_cpu'] = tmp

    tmp = config.get_string('/vgb-gui/preferences/gnuboy_location')
    if (tmp != None):
        gnuboy_config_opts['gnuboy_location'] = tmp
    tmp = config.get_bool('/vgb-gui/preferences/gnuboy_auto_gunzip')
    if (tmp != None):
        gnuboy_config_opts['gnuboy_auto_gunzip'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/gnuboy_scale')
    if (tmp != None):
        gnuboy_config_opts['gnuboy_scale'] = tmp
    tmp = config.get_string('/vgb-gui/preferences/gnuboy_density')
    if (tmp != None):
        gnuboy_config_opts['gnuboy_density'] = tmp

    tmp = config.get_string('/vgb-gui/preferences/savegame_location')
    if (tmp != None):
        config_opts['savegame_location'] = tmp
    tmp = config.get_vector('/vgb-gui/preferences/rom_path')
    if (tmp != None):
        # convert the tuple (tmp) to a list
        for i in tmp:
            config_opts['rom_path'].append(i)
    tmp = config.get_string('/vgb-gui/preferences/gb_emu')
    if (tmp != None):
        config_opts['gb_emu'] = tmp

def save_prefs():
    config.set_string('/vgb-gui/preferences/vgb_opt_uperiod', vgb_config_opts['vgb_opt_uperiod'])
    config.set_string('/vgb-gui/preferences/vgb_opt_gb_type', vgb_config_opts['vgb_opt_gb_type'])
    config.set_string('/vgb-gui/preferences/vgb_opt_sound', vgb_config_opts['vgb_opt_sound'])
    config.set_string('/vgb-gui/preferences/vgb_opt_sync', vgb_config_opts['vgb_opt_sync'])
    config.set_string('/vgb-gui/preferences/vgb_opt_scale', vgb_config_opts['vgb_opt_scale'])
    config.set_string('/vgb-gui/preferences/savegame_location', config_opts['savegame_location'])
    config.set_string('/vgb-gui/preferences/vgb_location', vgb_config_opts['vgb_location'])
    config.set_string('/vgb-gui/preferences/vgb_opt_autofire_A', vgb_config_opts['vgb_opt_autofire_A'])
    config.set_string('/vgb-gui/preferences/vgb_opt_autofire_B', vgb_config_opts['vgb_opt_autofire_B'])
    config.set_string('/vgb-gui/preferences/vgb_opt_pocket_printer', vgb_config_opts['vgb_opt_pocket_printer'])
    config.set_string('/vgb-gui/preferences/vgb_opt_washout', vgb_config_opts['vgb_opt_washout'])
    config.set_string('/vgb-gui/preferences/vgb_opt_shm', vgb_config_opts['vgb_opt_shm'])
    config.set_string('/vgb-gui/preferences/vgb_opt_save_cpu', vgb_config_opts['vgb_opt_save_cpu'])

    config.set_string('/vgb-gui/preferences/gnuboy_location', gnuboy_config_opts['gnuboy_location'])
    config.set_bool('/vgb-gui/preferences/gnuboy_auto_gunzip', gnuboy_config_opts['gnuboy_auto_gunzip'])
    config.set_string('/vgb-gui/preferences/gnuboy_scale', gnuboy_config_opts['gnuboy_scale'])
    config.set_string('/vgb-gui/preferences/gnuboy_density', gnuboy_config_opts['gnuboy_density'])

    config.set_vector('/vgb-gui/preferences/rom_path', config_opts['rom_path'])
    config.set_string('/vgb-gui/preferences/gb_emu', config_opts['gb_emu'])

    config.sync()

def main():
    global glade_loc

    glade_loc = ''

    load_prefs()
    
    # Get the directory from where VGB-GUI was executed
    glade_locs[1] = os.path.dirname (sys.argv[0]) + '/'

    initApp()

    add_gb_roms_to_games_list ()
    mainloop()


if __name__ == '__main__': main()
